#!/bin/bash

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m' # No Color

# Function to print status messages
print_status() {
	echo -e "${GREEN}[*] $1${NC}"
}

print_error() {
	echo -e "${RED}[!] $1${NC}"
	exit 1
}

# Check command line arguments
if [ $# -ne 1 ]; then
	print_error "Usage: $0 <php-version>\nExample: $0 8.3"
fi

PHP_VERSION="$1"
BUILD_DIR="$(pwd)"

# Function to detect extension name from config.m4 or config.w32
detect_extension_name() {
	local ext_name=""

	if [ -f "config.m4" ]; then
		ext_name=$(grep -E "PHP_(NEW|ADD)_EXTENSION\(" config.m4 | head -1 | sed -E 's/.*PHP_(NEW|ADD)_EXTENSION\(([^,]*).*/\2/' | tr -d '[:space:]')
	fi

	if [ -z "$ext_name" ] && [ -f "config.w32" ]; then
		ext_name=$(grep -E "ADD_EXTENSION\(" config.w32 | head -1 | sed -E 's/.*ADD_EXTENSION\(([^,]*).*/\1/' | tr -d '[:space:]')
	fi

	if [ -z "$ext_name" ]; then
		local c_files=(*.c)
		if [ -f "${c_files[0]}" ]; then
			ext_name=$(basename "${c_files[0]}" .c)
			print_status "Extension name detected from .c file: ${ext_name}"
		else
			print_error "Could not detect extension name. Please check your source files."
		fi
	fi

	echo "$ext_name"
}

# Check requirements
check_requirements() {
	print_status "Checking requirements..."
	if ! dpkg -l | grep -q "php${PHP_VERSION}-dev"; then
		print_error "php${PHP_VERSION}-dev is not installed. Please install it first."
	fi
}

# Main execution
main() {
	print_status "Starting NTS build process for PHP ${PHP_VERSION}"

	EXTENSION_NAME=$(detect_extension_name)
	print_status "Building extension: ${EXTENSION_NAME}"

	check_requirements

	# Clean up any existing build files
	make clean >/dev/null 2>&1
	/usr/bin/phpize${PHP_VERSION} --clean >/dev/null 2>&1

	# Remove autotools cache if it exists
	if [ -d "autom4te.cache" ]; then
		rm -rf autom4te.cache
	fi

	print_status "Running phpize${PHP_VERSION}..."
	/usr/bin/phpize${PHP_VERSION} || print_error "phpize failed"

	print_status "Running configure..."
	./configure --with-php-config=/usr/bin/php-config${PHP_VERSION} || print_error "configure failed"

	print_status "Running make..."
	make || print_error "make failed"

	# Create dist directory and copy the extension
	mkdir -p "${BUILD_DIR}/dist/nts"
	cp "modules/${EXTENSION_NAME}.so" "${BUILD_DIR}/dist/nts/" || print_error "Failed to copy extension"

	print_status "NTS build completed successfully!"
	#print_status "Extension can be found in: ${BUILD_DIR}/dist/nts/${EXTENSION_NAME}.so"
}

# Run the script
main
