#!/bin/bash

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m' # No Color

# Function to print status messages
print_status() {
	echo -e "${GREEN}[*] $1${NC}"
}

print_error() {
	echo -e "${RED}[!] $1${NC}"
	exit 1
}

# Check command line arguments
if [ $# -ne 2 ]; then
	print_error "Usage: $0 <php-version> <php-build-path>\nExample: $0 8.3 /tmp/phpbuild"
fi

PHP_VERSION="$1"
PHP_BUILD_PATH="$2"
BUILD_DIR="$(pwd)"
ZTS_PREFIX="${PHP_BUILD_PATH}/zts"
PHP_BUILD_DIR="${PHP_BUILD_PATH}/php"

# Create build directories
mkdir -p "${PHP_BUILD_DIR}" "${ZTS_PREFIX}"

# Function to build PHP with ZTS
build_php_zts() {
	local php_version="$1"
	local config_path="${ZTS_PREFIX}/bin/php-config"

	# All output to stderr
	{
		cd "${PHP_BUILD_DIR}" || print_error "Failed to enter PHP build directory"

		if [ ! -f "php-${php_version}.0.tar.gz" ]; then
			print_status "Downloading PHP ${php_version}.0 source..."
			wget "https://www.php.net/distributions/php-${php_version}.0.tar.gz" -O "php-${php_version}.0.tar.gz" || print_error "Failed to download PHP source"
		fi

		print_status "Extracting PHP source..."
		tar xzf "php-${php_version}.0.tar.gz" || print_error "Failed to extract PHP source"
		cd "php-${php_version}.0" || print_error "Failed to enter PHP source directory"

		print_status "Configuring PHP with ZTS..."
		./configure \
			--enable-zts \
			--prefix="${ZTS_PREFIX}" \
			--enable-debug \
			|| print_error "Failed to configure PHP"

		print_status "Building PHP..."
		make -j$(nproc) || print_error "Failed to build PHP"
		print_status "Installing PHP..."
		make install || print_error "Failed to install PHP"
	} >&2

	# Return the php-config path
	echo "$config_path"
}

# Function to detect extension name from config.m4 or config.w32
detect_extension_name() {
	local ext_name=""

	if [ -f "config.m4" ]; then
		ext_name=$(grep -E "PHP_(NEW|ADD)_EXTENSION\(" config.m4 | head -1 | sed -E 's/.*PHP_(NEW|ADD)_EXTENSION\(([^,]*).*/\2/' | tr -d '[:space:]')
	fi

	if [ -z "$ext_name" ] && [ -f "config.w32" ]; then
		ext_name=$(grep -E "ADD_EXTENSION\(" config.w32 | head -1 | sed -E 's/.*ADD_EXTENSION\(([^,]*).*/\1/' | tr -d '[:space:]')
	fi

	if [ -z "$ext_name" ]; then
		local c_files=(*.c)
		if [ -f "${c_files[0]}" ]; then
			ext_name=$(basename "${c_files[0]}" .c)
			print_status "Extension name detected from .c file: ${ext_name}"
		else
			print_error "Could not detect extension name. Please check your source files."
		fi
	fi

	echo "$ext_name"
}

# Main execution
main() {
	print_status "Starting ZTS build process for PHP ${PHP_VERSION}"

	EXTENSION_NAME=$(detect_extension_name)
	print_status "Building extension: ${EXTENSION_NAME}"

	# First build PHP with ZTS
	print_status "Building PHP ${PHP_VERSION} with ZTS support..."
	local zts_php_config
	zts_php_config=$(build_php_zts "${PHP_VERSION}")
	print_status "ZTS PHP build completed, config at: ${zts_php_config}"

	# Clean up any existing build files
	cd "${BUILD_DIR}" || print_error "Failed to return to build directory"
	make clean >/dev/null 2>&1
	"${ZTS_PREFIX}/bin/phpize" --clean >/dev/null 2>&1

	# Remove autotools cache if it exists
	if [ -d "autom4te.cache" ]; then
		rm -rf autom4te.cache
	fi

	print_status "Using ZTS PHP from: ${ZTS_PREFIX}"

	print_status "Running phpize..."
	"${ZTS_PREFIX}/bin/phpize" || print_error "phpize failed"

	print_status "Running configure..."
	./configure --with-php-config="${zts_php_config}" || print_error "configure failed"

	print_status "Running make..."
	make || print_error "make failed"

	# Create dist directory and copy the extension
	mkdir -p "${BUILD_DIR}/dist/zts"
	cp "modules/${EXTENSION_NAME}.so" "${BUILD_DIR}/dist/zts/" || print_error "Failed to copy extension"

	print_status "ZTS build completed successfully!"
	#print_status "Extension can be found in: ${BUILD_DIR}/dist/zts/${EXTENSION_NAME}.so"
}

# Cleanup function
cleanup() {
	if [ -d "${PHP_BUILD_DIR}" ]; then
		print_status "Cleaning up PHP build directory..."
		rm -rf "${PHP_BUILD_DIR}"
	fi
}

# Trap for cleanup
trap cleanup EXIT

# Run the script
main
